//--------------------------------------------------------------------
//--------------------------------------------------------------------
//--------------------------------------------------------------------

#include rmPaintBox.h  

Class rmToolBar(oP) of Container(oP) Custom

   this.Version = "1.06.1012"

   /*

	Classname:  rmToolBar
    
   Date:       Nov 2005

   For:        dBASE PLUS / dB2K / Visual dBase 7.** 
               Windows 95/98/ME/XP/NT/2000

   Author:     Ronnie MacGregor

   Contact:    rmToolBarFeedBack@dBASEdeveloper.co.uk

   Copyright:  Ronnie MacGregor. All rights reserved.

               This control may be used free of charge by dBASE
               developers, but must be credited to the author.

   Disclaimer: It is the duty of the user to determine the suitability
               of this code for use. In no circumstance will the author
               accept any responsibility for any adverse consequence 
               arising from the use of this code.
	
   
   Description:

      A custom PaintBox based ToolBar.


   Revisions:

      None.


   Methods:

      SetBarColour(cColour)

         This method is used to specifically define the XP colour scheme to use,
         and accepts a parameter from the following :

            "Default (blue)" "Blue"

            "Metallic" "Silver"

            "Homestead" "Olive Green"

            "BlackGlass"

            "OSX"

         Note that if running on XP, this.OverRideThemeColour must be true
         in order to see the results of this setting rather than the XP colour
         in force.


   Useage:

      See rmToolbar.wfm for sample code.


   Dependancies:

      rmPaintbox.cc  - Base paintbox class

      rmPaintbox.h   - Header file

      rmToolBar.dll  - Bitmap resource
      

   Acknowledgements:

      None.  


   Revisions :

      2006/03/19

         Jan Hoelterling reported that this control would not run on dB2K 0.4.

         Problem 1:
         This proved to be a problem with the order in which events are
         occuring "under the hood" in dB2K which is not evident in dBASE Plus.
         The control was getting its initial paint before it had finished
         initialising, and did not hve its hModule property set.
         This was cured by initialising the hModule custom property to 0
         allowing the PaintBitmapStretch etc. API calls to fail silently
         until such time as this property holds a valid handle.

         Problem 2:
         The paintbox did not get a speedtip property until around Plus 2.01.
         This has been cured by try/catching any code dealing with speedtips.

      2006/09/30

         Updated to provide automatic changes of toolbar colour to match
         a change of XP theme colour.

         
   */

   // -------------------------------------

   // File dependency
   if "RUNTIME" $ upper(version(1)) == true
      set procedure to rmPaintbox.cc additive
   else
      try
         set procedure to :rmPaintbox:rmPaintbox.cc additive
      catch (exception e)
         try
            set procedure to rmPaintbox.cc additive
         catch (exception e)
            msgbox("Required file - rmPaintbox.cc not found." + chr(13) ;
                 + chr(13) ;
                 + "Either the source alias to this file does not exist," ;
                 + chr(13) ;
                 + "or the file is not found in the current directory." ;
                 , " Error", 16 +0)
         endtry
      endtry
   endif

   // -------------------------------------

   // Stock Properties
   with (this)
      transparent = true
      colornormal = ""
      borderStyle = 3	// None
      width = 200
      height = 25
      pageno = 0
   endwith

   // -------------------------------------

   // Custom Properties                                                                      
   this.Resource = "" 
   this.NextObjectLeft = 3
   this.NextObjectNumber = 1
   this.OverRideThemeColour = false
   this.BarColour = ""
   this.DefaultBarColour = "METALLIC" // "BLUE" "HOMESTEAD" "METALLIC" "BLACKGLASS" "OSX"
   this.Disabled = false
   this.aObjectRefs = new array()
   this.RoundEnds = true
   this.Rowset = null

   // -------------------------------------

   // Create toolbar background control
   this.BarArea = new rmPaintBox(this)
   with (this.BarArea)
      onOpen = CLASS::BarAreaOnOpen
      left = 0
      top = 0
      width = 800
      height = 25
   endwith

   // -------------------------------------

   function BarAreaOnOpen()

      local bRet

      // Adjust bar area to fit 
      this.width := this.parent.width
      this.height := this.parent.height

      // Start up theme awareness
      bRet = this.StartThemeAwareness()
      if bRet == true  
         // Update BarColour property to match current XP theme colour
         this.parent.BarColour := this.ThemeColorName
      endif

      // Load Bitmap Resource dll
      this.LoadDLL(this.parent.Resource)
       
      // Initiate onPaint
      this.onPaint = class::BarAreaOnPaint

   return

   // -------------------------------------

   function SetBarColour(cColour)

      cColour := upper(cColour) 

      // Set colour property
      do case

         case cColour == "METALLIC" or cColour == "SILVER"
            this.DefaultBarColour := "METALLIC"

         case cColour == "HOMESTEAD" or cColour == "OLIVE GREEN"
            this.DefaultBarColour := "HOMESTEAD"

         case cColour == "BLACKGLASS" or cColour == "BLACK GLASS"
            this.DefaultBarColour := "BLACKGLASS"

         case cColour == "OSX" 
            this.DefaultBarColour := "OSX"

         otherwise
            this.DefaultBarColour := "BLUE"

      endcase

   return

   // -------------------------------------

   function BarAreaOnPaint()

      // Update toolbar colour if required
      if this.ThemeAware == true and this.parent.OverRideThemeColour == false
         this.parent.BarColour := this.ThemeColorName
      else
         this.parent.BarColour := this.parent.DefaultBarColour
      endif

      local hDC, hDefPen, hDefBrush, xPaint, tw, th, cBMPName 

      this.width = this.parent.width           
      this.height = this.parent.height   
      tw = this.width
      th = this.height
       
      xPaint = space(32) // throw-away buffer
      
      // invalidate complete control to make repainting ready
      invalidateRect(this.hWND, 0, true)

      // Start the paint run
      hDC = beginpaint(this.hWND, xPaint)

      if this.parent.RoundEnds == true

         // Left end
         cBMPName = this.parent.BarColour + "_TOOLBARLEFT_BMP"
         this.PaintTransparentBitmap(cBMPName, hDC, 0, 0, "800080")

         // Main Bar
         cBMPName = this.parent.BarColour + "_TOOLBARBACK_BMP"
         this.PaintBitmapStretch(cBMPName, hDC, 4, 0, tw-8, 25, 0, 0, 4, 25)

         // Right end
         cBMPName = this.parent.BarColour + "_TOOLBARRIGHT_BMP"
         this.PaintTransparentBitmap(cBMPName, hDC, tw-4, 0, "800080")

      else

         // Main Bar
         cBMPName = this.parent.BarColour + "_TOOLBARBACK_BMP"
         this.PaintBitmapStretch(cBMPName, hDC, 0, 0, tw, 25, 0, 0, 4, 25)

      endif

      // Finish Painting
      EndPaint(this.hWND,xPaint) 
   
   return

   // -------------------------------------

   function CreateButton(cName, cResource, cBackgroundName, cIconName, nLeft, bDisabled, bToggle, bToggledDown, cRadioGroup, cSpeedTip, cClickCode)

      local oRef 

      private sMacString

      // Create the Button
      if cName == ""
			cName = "Button_" + cIconName + "_" + this.NextObjectNumber 
         this.NextObjectNumber ++
      endif
      sMacString = "this." + cName + " = new rmToolBarButton(this)"
      &sMacString.

      // Get object ref to button
      sMacString = "this." + cName
      oRef = &sMacString.

      // Set button properties
      oRef.Resource := cResource
      oRef.BackgroundName := cBackgroundName
      oRef.IconName := cIconName
      if nLeft > 0
         oRef.Left := nLeft
      else
         oRef.Left := this.NextObjectLeft
      endif
      oRef.Disabled := bDisabled
      oRef.Toggle := bToggle
      oRef.ToggledDown := bToggledDown
      oRef.RadioGroup := cRadioGroup
      // Paintbox only has speedtip dBASE Plus 2.01 onwards
      // so don't use asignment operator and a custom property
      // will be created if need be.
      oRef.SpeedTip = cSpeedTip 

      // Set up onClick functionality
      if not empty(cClickCode)
         sMacString = "this." + cName + ".onMiddleDblClick = {; " + cClickCode + "}"
         &sMacString.
      endif 

      // Add to array of object references
      this.aObjectRefs.add(oRef)

      // Load Bitmap Resource dll
      oRef.LoadDLL(cResource)

      // Update default position for next object
      this.NextObjectLeft := oRef.left + 23

   return

   // -------------------------------------

   function CreateSeparator(cName, cResource, nLeft)

      local oRef 

      private sMacString

      // Create the Separator
      if cName == ""
			cName = "Separator_" + this.NextObjectNumber          
         this.NextObjectNumber ++
      endif
      sMacString = "this." + cName + " = new rmToolBarSEP(this)"
      &sMacString.

      // Get object ref to separator
      sMacString = "this." + cName
      oRef = &sMacString.

      // Set button properties
      oRef.Resource := cResource
      if nLeft > 0
         oRef.Left := nLeft
      else
         oRef.Left := this.NextObjectLeft
      endif

      // Add to array of object references
      this.aObjectRefs.add(oRef)

      // Load Bitmap Resource dll
      oRef.LoadDLL(cResource)

      // Update default position for next object
      this.NextObjectLeft := oRef.left + 8

   return

   // -------------------------------------

   function CreateEntryField(cName, cResource, cBackgroundName, nLeft, nWidth, bNoRightEnd, bDisabled, cSpeedTip, cClickCode)

      local oRef 

      private sMacString

      // Create the Background paintbox for the Entryfield
      if cName == ""
			cName = "EntryField_" + this.NextObjectNumber           
         this.NextObjectNumber ++
      endif
      sMacString = "this." + cName + " = new rmToolBarEF(this)"
      &sMacString.

      // Get object ref to button
      sMacString = "this." + cName
      oRef = &sMacString.

      // Set properties
      oRef.Resource := cResource
      oRef.BackgroundName := cBackgroundName
      if nLeft > 0
         oRef.Left := nLeft
      else
         oRef.Left := this.NextObjectLeft
      endif
      oRef.Width := nWidth
      oRef.NoRightEnd := bNoRightEnd
      oRef.Disabled := bDisabled
      // Paintbox only has speedtip dBASE Plus 2.01 onwards
      // so don't use asignment operator and a custom property
      // will be created if need be.
      oRef.SpeedTip = cSpeedTip 
      
      // Set up Enter or return key functionality   
      if not empty(cClickCode)
         sMacString = "this." + cName + ".onMiddleDblClick = {; " + cClickCode + "}"
         &sMacString.
      endif 

      // Add to array of object references
      this.aObjectRefs.add(oRef)

      // Manually fire the onOpen since the form is already open
      oRef.onOpen() 

      // Load Bitmap Resource dll
      oRef.LoadDLL(cResource)

      // Update default position for next object
      this.NextObjectLeft := oRef.left + oRef.width

   return

   // -------------------------------------

   function CreateText(cName, cResource, cBackgroundName, nLeft, nWidth, cText, cFontName, nFontSize, bFontBold, bFontItalic, bEmboss, cSpeedTip)

      local oRef 

      private sMacString

      // Create the PaintBox Text object
      if cName == ""
			cName = "Text_" + this.NextObjectNumber          
         this.NextObjectNumber ++
      endif
      sMacString = "this." + cName + " = new rmToolBarTXT(this)"
      &sMacString.

      // Get object ref to button
      sMacString = "this." + cName
      oRef = &sMacString.

      // Set properties
      oRef.Resource := cResource
      oRef.BackgroundName := cBackgroundName
      if nLeft > 0
         oRef.Left := nLeft
      else
         oRef.Left := this.NextObjectLeft
      endif
      oRef.Width := nWidth
      oRef.Text := cText
      if not empty(cFontName) 
         oRef.FontName := cFontName
      endif
      if not empty(nFontSize) 
         oRef.FontSize := nFontsize
      endif
      oRef.FontBold := bFontBold
      oRef.FontItalic := bFontItalic
      oRef.Emboss := bEmboss
      // Paintbox only has speedtip dBASE Plus 2.01 onwards
      // so don't use asignment operator and a custom property
      // will be created if need be.
      oRef.SpeedTip = cSpeedTip 

      // Add to array of object references
      this.aObjectRefs.add(oRef)

      // Load Bitmap Resource dll
      oRef.LoadDLL(cResource)

      // Update default position for next object
      this.NextObjectLeft := oRef.left + oRef.width

   return

   // -------------------------------------

   function Refresh()

      // Repaint Bar
      this.BarArea.onPaint()

      // Repaint all toolbar objects
      local nCount
      for nCount = 1 to this.aObjectRefs.size
          this.aObjectRefs[nCount].onPaint()
      endfor // next

   return

   // -------------------------------------

   function MoveElementLeft(nElementNumber)

      local oTemp 

      // Validate
      if nElementNumber == 1 or this.aObjectRefs.size < 2
         return false
      endif

      // Change Left values of controls
      this.aObjectRefs[nElementNumber].Left := this.aObjectRefs[nElementNumber -1].Left
      this.aObjectRefs[nElementNumber -1].Left += this.aObjectRefs[nElementNumber].Width
                                         
      // Swap object ref positions in array
      oTemp = this.aObjectRefs[nElementNumber -1]
      this.aObjectRefs[nElementNumber -1] := this.aObjectRefs[nElementNumber]
      this.aObjectRefs[nElementNumber] := oTemp

      // Repaint
      this.Refresh()

   return true
   
   // -------------------------------------

   function MoveElementRight(nElementNumber)

      local oTemp 

      // Validate
      if nElementNumber == this.aObjectRefs.size or this.aObjectRefs.size < 2
         return false
      endif

      // Change Left values of controls
      this.aObjectRefs[nElementNumber +1].Left := this.aObjectRefs[nElementNumber].Left
      this.aObjectRefs[nElementNumber].Left += this.aObjectRefs[nElementNumber +1].Width
                                         
      // Swap object ref positions in array
      oTemp = this.aObjectRefs[nElementNumber +1]
      this.aObjectRefs[nElementNumber +1] := this.aObjectRefs[nElementNumber]
      this.aObjectRefs[nElementNumber] := oTemp

      // Repaint
      this.Refresh()

   return true

   // -------------------------------------

   function DeleteElement(nElementNumber)

      // Validate
      if not (nElementNumber > 0 and nElementNumber <= this.aObjectRefs.size)
         return false
      endif

      local nCount, nWidth

      // Change Left values of subsequent controls
      nWidth = this.aObjectRefs[nElementNumber].width
      for nCount = nElementNumber + 1 to this.aObjectRefs.size 
            this.aObjectRefs[nCount].Left := this.aObjectRefs[nCount].Left - nWidth
      endfor // next

      // Update default position for next object
      this.NextObjectLeft := this.NextObjectLeft - nWidth

      // Delete Element
      if this.aObjectRefs[nElementNumber].classname == "RMTOOLBAREF"
         release object this.aObjectRefs[nElementNumber].EF
      endif
      release object this.aObjectRefs[nElementNumber]
      this.aObjectRefs[nElementNumber] = null

      // Move subsequent Elements up the array
      for nCount = nElementNumber to this.aObjectRefs.size - 1
         this.aObjectRefs[nCount] = this.aObjectRefs[nCount + 1] 
      endfor // next

      // Resize array
      if this.aObjectRefs.size > 1
         this.aObjectRefs.resize(this.aObjectRefs.size - 1)
      else
         release object this.aObjectRefs
         this.aObjectRefs = new array()
      endif

      // Repaint
      this.Refresh()

   return true

   // -------------------------------------

   function DestroyToolBar()

      local nCount

      if this.aObjectRefs.size > 0
         for nCount = 1 to this.aObjectRefs.size

            // Delete Element
            if this.aObjectRefs[nCount].classname == "RMTOOLBAREF"
               release object this.aObjectRefs[nCount].EF
            endif
            release object this.aObjectRefs[nCount]
            this.aObjectRefs[nCount] = null

         endfor
      endif

      release object this

   return

   // -------------------------------------

   function StringToClassName(cString)

      local nCount

      // Replace any spaces or slashes with underscore
      for nCount = 1 To len(cString)
         if Substr(cString, nCount, 1) $ ' /'
            cString = Stuff(cString, nCount, 1, '_')
         endif
      endfor //Next

      // Prepend leading digit with underscore
      if Substr(cString, 1, 1) $ '0123456789'
         cString = "_" + cString
      endif

      // Limit to dBASE max of 32 chars
      if len(cString) > 32
         cString = substr(cString, 1, 32)
      endif

   return cString

   // -------------------------------------

   function StreamToolBar(cFileName, cFilePath, cClassName)

      // Validate
      if empty(cFileName)
         msgbox("No File Name for new Toolbar supplied.", " Error", 16 + 0)
         return false
      endif
      if empty(cClassName)
         cClassName = this.StringToClassName((cFileName))
      endif

      local nCount, aO, f, cFullFileName

      aO = this.aObjectRefs
              
      // Ensure numbers are output with correct decimal point
      cOldPoint = set("point")
      set point to "."

      // Build full file name
      if not empty(argvector(2))
         if not rat("\", cFilePath ,1 ) == 1
            if len(trim(cFilePath)) > 0
               cFilePath = cFilePath + "\"
            endif
         endif
         cFullFileName = cFilePath + cFileName + ".cc"               
      else
         cFullFileName = cFileName + ".cc"
      endif

      // Create output File
      f = new File()
      f.create(cFullFileName)   

      // Deal with File Header


      f.puts('//--------------------------------------------------------------------')
      f.puts('//--------------------------------------------------------------------')
      f.puts('//--------------------------------------------------------------------')
      f.puts('')
      f.puts('#include rmPaintBox.h')
      f.puts('')
      f.puts('class ' + cClassName + '(oP) of rmToolbar(oP) from rmToolBar.cc')
      f.puts('')
      f.puts('')
      f.puts('   /*')
      f.puts('   ==========================================================')
      f.puts('')
      f.puts('')
      f.puts('   Auto-Generated ToolBar Code')
      f.puts('')
      f.puts('')
      f.puts('   ' + cClassName + '.cc')
      f.puts('')
      f.puts('   A SubClass of rmToolBar.cc')
      f.puts('')
      f.puts('   Created on ' + date() + ' at ' + time())
      f.puts('')
      f.puts('')
      f.puts('   Copyright  2005 - ' + year(date()))
      f.puts('')
      f.puts('   Ronnie MacGregor')
      f.puts('')
      f.puts('   www.dBASEdeveloper.com')
      f.puts('')
      f.puts('   All rights reserved.')
      f.puts('')
      f.puts('')
      f.puts('   Quick and easy ToolBars for dBASE applications.')
      f.puts('')
      f.puts('')
      f.puts('   ==========================================================')
      f.puts('   */')
      f.puts('')
      f.puts('   //----------------------')
      f.puts('')
      f.puts('   // Stock Properties')
      f.puts('   with (this)')
      f.puts('      left = ' + this.left)
      f.puts('      top = ' + this.top)
      f.puts('      width = ' + this.width)
      f.puts('   endwith')
      f.puts('')
      f.puts('   // Custom Properties')
      f.puts('   this.Resource = "' + this.Resource + '"')
      f.puts('   this.NextObjectLeft = ' + this.NextObjectLeft)
      f.puts('   this.BarColour = ""')
      f.puts('   this.DefaultBarColour = "' + this.DefaultBarColour + '"')
      f.puts('   this.Disabled = ' + this.Disabled)
      f.puts('   this.RoundEnds = ' + this.RoundEnds)
      f.puts('   this.OverRideThemeColour = ' + this.OverRideThemeColour)
      f.puts('')
      f.puts('   //----------------------')
      f.puts('')
      f.puts('')

      for nCount = 1 to aO.size

         do case

            case aO[nCount].ClassName == "RMTOOLBARBUTTON"

               f.puts('   this.' + aO[nCount].Name + ' = new rmToolBarButton(this)')
               f.puts('   with (this.' + aO[nCount].Name + ')')
               f.puts('      Resource = "' + aO[nCount].Resource + '"')
               f.puts('      BackgroundName = "' + aO[nCount].BackgroundName + '"')
               f.puts('      IconName = "' + aO[nCount].IconName + '"')
               f.puts('      Left = ' + aO[nCount].Left)
               f.puts('      Width = ' + aO[nCount].Width)   
               f.puts('      Disabled = ' + aO[nCount].Disabled)   
               f.puts('      Toggle = ' + aO[nCount].Toggle)   
               f.puts('      ToggledDown = ' + aO[nCount].ToggledDown)
               f.puts('      RadioGroup = "' + aO[nCount].RadioGroup + '"')
               f.puts('      SpeedTip = "' + aO[nCount].SpeedTip + '"')
               if len("" + aO[nCount].onMiddleDblClick) > 0
                  f.puts('      onMiddleDblClick = ' + aO[nCount].onMiddleDblClick)
               endif
               f.puts('   endwith')
               f.puts('   oRef = this.' + aO[nCount].Name)   
               f.puts('   oRef.LoadDLL(oRef.Resource)')   
               f.puts('   this.aObjectRefs.add(oRef)') 
               f.puts('   this.NextObjectLeft := oRef.left + oRef.width')
               f.puts('')
               f.puts('')

            case aO[nCount].ClassName == "RMTOOLBARSEP"

               f.puts('   this.' + aO[nCount].Name + ' = new rmToolBarSEP(this)')
               f.puts('   with (this.' + aO[nCount].Name + ')')
               f.puts('      Resource = "' + aO[nCount].Resource + '"')
               f.puts('      Left = ' + aO[nCount].Left)
               f.puts('   endwith')
               f.puts('   oRef = this.' + aO[nCount].Name)   
               f.puts('   oRef.LoadDLL(oRef.Resource)')   
               f.puts('   this.aObjectRefs.add(oRef)') 
               f.puts('   this.NextObjectLeft := oRef.left + oRef.width')
               f.puts('')
               f.puts('')

            case aO[nCount].ClassName == "RMTOOLBAREF"

               f.puts('   this.' + aO[nCount].Name + ' = new rmToolBarEF(this)')
               f.puts('   with (this.' + aO[nCount].Name + ')')
               f.puts('      Resource = "' + aO[nCount].Resource + '"')
               f.puts('      BackgroundName = "' + aO[nCount].BackgroundName + '"')
               f.puts('      Left = ' + aO[nCount].Left)
               f.puts('      Width = ' + aO[nCount].Width)
               f.puts('      NoRightEnd = ' + aO[nCount].NoRightEnd)
               f.puts('      Disabled = ' + aO[nCount].Disabled)
               f.puts('      SpeedTip = "' + aO[nCount].SpeedTip + '"')
               if len("" + aO[nCount].onMiddleDblClick) > 0
                  f.puts('      onMiddleDblClick = ' + aO[nCount].onMiddleDblClick)
               endif                  
               f.puts('   endwith')
               f.puts('   oRef = this.' + aO[nCount].Name)   
               f.puts('   oRef.LoadDLL(oRef.Resource)')   
               f.puts('   this.aObjectRefs.add(oRef)') 
               f.puts('   this.NextObjectLeft := oRef.left + oRef.width')
               f.puts('')
               f.puts('')

            case aO[nCount].ClassName == "RMTOOLBARTXT"

               f.puts('   this.' + aO[nCount].Name + ' = new rmToolBarTXT(this)')
               f.puts('   with (this.' + aO[nCount].Name + ')')
               f.puts('      Resource = "' + aO[nCount].Resource + '"')
               f.puts('      BackgroundName = "' + aO[nCount].BackgroundName + '"')
               f.puts('      Left = ' + aO[nCount].Left)
               f.puts('      Width = ' + aO[nCount].Width)
               f.puts('      Text = "' + aO[nCount].Text + '"')
               f.puts('      FontName = "' + aO[nCount].FontName + '"')
               f.puts('      FontSize = ' + aO[nCount].FontSize)
               f.puts('      FontBold = ' + aO[nCount].FontBold)
               f.puts('      FontItalic = ' + aO[nCount].FontItalic)
               f.puts('      Emboss = ' + aO[nCount].Emboss)
               f.puts('      SpeedTip = "' + aO[nCount].SpeedTip + '"')
               f.puts('   endwith')
               f.puts('   oRef = this.' + aO[nCount].Name)   
               f.puts('   oRef.LoadDLL(oRef.Resource)')   
               f.puts('   this.aObjectRefs.add(oRef)') 
               f.puts('   this.NextObjectLeft := oRef.left + oRef.width')
               f.puts('')
               f.puts('')                  

         endcase
  
      endfor // next

      // Finish Class definition

      f.puts('endclass') 
      f.puts('')
      f.puts('')
      f.puts('// eof')

      // Close file
      f.close() 
      f = null

      // Restore point setting
      set point to cOldPoint

   return true

   // -------------------------------------

endclass

//--------------------------------------------------------------------
//--------------------------------------------------------------------
//--------------------------------------------------------------------

Class rmToolBarButton(oP) of rmPaintbox(oP) // from "rmPaintbox.cc"

   with (this)
      onPaint = class::ToolButtonOnPaint
      top = 1
      width = 23
      height = 22
   endwith

   // -------------------------------------

   // Custom properties

   this.Resource = ""
   this.BackgroundName = ""
   this.IconName = ""
   this.Toggle = false
   this.ToggledDown = false
   this.RadioGroup = ""
   this.Disabled = false
   this.IsMouseOver = false
   this.IsMouseDown = false
   this.hModule = 0

   // -------------------------------------

   function ToolButtonOnPaint()

      // Update toolbar colour if required
      if this.parent.BarArea.ThemeAware == true and this.parent.OverRideThemeColour == false
         this.parent.BarColour := this.parent.BarArea.ThemeColorName
      else
         this.parent.BarColour := this.parent.DefaultBarColour
      endif

      local hDC, hDefPen, hDefBrush, xPaint, tw, th, cBMPName 

      tw = this.width
      th = this.height
      
      xPaint = space(32) // throw-away buffer
      
      // invalidate complete control to make repainting ready
      invalidateRect(this.hWND, 0, true)

      // Start the paint run
      hDC = beginpaint(this.hWND, xPaint)

      // Paint Background
      cBMPName = this.parent.BarColour + "_TOOLBARBACK_BMP"
      this.PaintBitmapStretch(cBMPName, hDC, 0, 0, 23, 22, 0, 1, 20, 22)  

      // Paint Icon
      if this.BackgroundName == ""
         cBackgroundName = "" 
      else
         cBackgroundName = this.parent.BarColour + "_" + this.BackgroundName            
      endif

      cBMPName = this.IconName

      do case

         // Disabled - state 3
         case this.Disabled == true

            this.PaintTransparentSubBitmap(cBMPName, hDC, 0, 0, "800080", 6, 3)

         // Selected (up down or toggled down) - state 4 and 5
         case this.ToggledDown == true

            if this.IsMouseDown == true
  
               this.PaintTransparentSubBitmap(cBackgroundName, hDC, 0, 0, "800080", 6, 5)
               this.PaintTransparentSubBitmap(cBMPName, hDC, 0, 0, "800080", 6, 5)

            else

               this.PaintTransparentSubBitmap(cBackgroundName, hDC, 0, 0, "800080", 6, 4)
               this.PaintTransparentSubBitmap(cBMPName, hDC, 0, 0, "800080", 6, 4)

            endif

         // Down (clicked) - state 2
         case this.IsMouseDown == true

            this.PaintTransparentSubBitmap(cBackgroundName, hDC, 0, 0, "800080", 6, 2)
            this.PaintTransparentSubBitmap(cBMPName, hDC, 0, 0, "800080", 6, 2)

         // Normal (hot if required)
         otherwise 

            if this.IsMouseOver == true and this.ToggledDown == false
             
               // Hot - state 0
               this.PaintTransparentSubBitmap(cBackgroundName, hDC, 0, 0, "800080", 6, 0)
               this.PaintTransparentSubBitmap(cBMPName, hDC, 0, 0, "800080", 6, 0)
               
            else

               // Normal - state 1
               this.PaintTransparentSubBitmap(cBackgroundName, hDC, 0, 0, "800080", 6, 1)
               this.PaintTransparentSubBitmap(cBMPName, hDC, 0, 0, "800080", 6, 1)

            endif

      endcase

      // Finish Painting
      EndPaint(this.hWND,xPaint) 
   
   return

   // -------------------------------------

   function onMouseMove(flags, col, row)

      if this.Disabled == false

         if col > this.width or row > this.height or col < 0 or row < 0
            this.IsMouseOver := false
            this.IsMouseDown = false
            ReleaseCapture()
            class::ToolButtonOnPaint()
         else
            SetCapture(this.hwnd)
            if not this.IsMouseOver == true
               this.IsMouseOver := true
               class::ToolButtonOnPaint()
            endif
         endif

      endif

   return

   // -------------------------------------

   function onLeftMousedown()

      if this.Disabled == false

         local tp, nCount

         tp = this.parent

         this.setFocus() 
               
         this.IsMouseDown := true
         this.IsMouseOver := False

         // Deal with toggle state
         if this.Toggle == true
            if this.RadioGroup == ""
               this.ToggledDown := not this.ToggledDown
            elseif this.ToggledDown == false
               this.ToggledDown := not this.ToggledDown
            endif
         endif

         // Deal with Radio Buttons
         if not this.RadioGroup == ""
            for nCount = 1 to tp.aObjectRefs.size 
               if tp.aObjectRefs[nCount].ClassName == "RMTOOLBARBUTTON"
                  if tp.aObjectRefs[nCount].RadioGroup == this.RadioGroup and not tp.aObjectRefs[nCount].Name == this.Name
                     tp.aObjectRefs[nCount].ToggledDown := false
                     tp.aObjectRefs[nCount].onPaint()
                  endif
               endif
            endfor // next
         endif

         // Paint this button
         class::ToolButtonOnPaint()

      endif

   return

   // -------------------------------------

   function onLeftMouseUp()

      if this.Disabled == false

         this.IsMouseOver := true
         this.IsMouseDown := false

         SetCapture(this.hwnd)

         class::ToolButtonOnPaint()

         // Run onClick code
         if this.Disabled == false
            try
               this.onMiddleDblClick()
            catch (exception e) 
               // No click event defined ?
            endtry
         endif

      endif

   return

   // -------------------------------------

   function ReleaseMouse()

      /*
      The rmToolButton captures the mouse messages until such time as a mouse
      move shows that the mouse is no longer over the button.

      This can cause problems if the buttons "onClick" code fires up, for 
      example, a modal dialogue which takes focus, preventing mousemove messages
      getting to the rmToolButton and allowing the release of the mouse.

      In this circumstance, this function can be called from the "onCLick" code
      prior to initiating the modal dialogue to force the release of the mouse.
      */

      this.IsMouseOver := false
      ReleaseCapture()
      this.onPaint()

   return

   // -------------------------------------

endclass

//--------------------------------------------------------------------
//--------------------------------------------------------------------
//--------------------------------------------------------------------

Class rmToolBarSEP(oF) of rmPaintbox(oF) // from "rmPaintbox.cc"

   with (this)
      onPaint = class::TBSonPaint
      top = 1
      width = 8
      height = 22
   endwith

   // -------------------------------------

   // Custom properties
   this.Resource = ""
   this.hModule = 0

   // -------------------------------------

   function TBSonPaint()

      // Update toolbar colour if required
      if this.parent.BarArea.ThemeAware == true and this.parent.OverRideThemeColour == false
         this.parent.BarColour := this.parent.BarArea.ThemeColorName
      else
         this.parent.BarColour := this.parent.DefaultBarColour
      endif

      local hDC, hDefPen, hDefBrush, xPaint, tw, th, cBMPName 

      tw = this.width
      th = this.height
       
      xPaint = space(32) // throw-away buffer
      
      // invalidate complete control to make repainting ready
      invalidateRect(this.hWND, 0, true)

      // Start the paint run
      hDC = beginpaint(this.hWND, xPaint)

      // Paint Background
      cBMPName = this.parent.BarColour + "_TOOLBARBACK_BMP"
      this.PaintBitmapStretch(cBMPName, hDC, 0, 0, 23, 22, 0, 1, 20, 22)

      // Paint Icon
      cBMPName = this.parent.BarColour + "_SEPARATOR_BMP"
      this.PaintTransparentBitmap(cBMPName, hDC, 0, 0, "800080")

      // Finish Painting
      EndPaint(this.hWND,xPaint) 
   
   return

   // -------------------------------------

endclass

//--------------------------------------------------------------------
//--------------------------------------------------------------------
//--------------------------------------------------------------------

Class rmToolBarEF(oF) of rmPaintbox(oF) // from "rmPaintbox.cc"

   with (this)
      onOpen = class::TBEF_onOpen
      onLeftMousedown = class::TBEF_onLeftMousedown
      onPaint = class::TBEF_onPaint
      top = 1
      width = 23
      height = 22
   endwith

   // -------------------------------------

   // Custom properties
   this.Resource = ""
   this.BackgroundName = ""
   this.Disabled = false
   this.IsMouseOver = false
   this.IsSelected = false
   this.NoRightEnd = false
   this.hModule = 0

   // -------------------------------------

   // Create overlayed EntryField control
   this.EF = new ENTRYFIELD(this.parent)
   with (this.EF)
      onGotFocus = {; this.Owner.IsSelected := true; this.Owner.onPaint()}
      onLostFocus = {; this.Owner.IsSelected := false; this.Owner.onPaint()}
      key = CLASS::EF_Key
      top = 5
      left = -100 
      width = 0   
      height = 14
      function = "I99999"
      maxLength = 5      // Must be defined after "function"
      border = false
      value = null   
      colorNormal = "black/white"
      borderStyle = 3   // None
      fontName = "Verdana"
      fontSize = 9.0
   endwith

   // -------------------------------------

   function TBEF_onOpen()

      // Set properties that can't be set before open
      this.EF.Owner = this
      this.EF.left := this.left + 3
      this.EF.width := this.width - 6
      this.EF.speedTip := (this.SpeedTip) 

   return

   // -------------------------------------

   function TBEF_onPaint()

      // Update toolbar colour if required
      if this.parent.BarArea.ThemeAware == true and this.parent.OverRideThemeColour == false
         this.parent.BarColour := this.parent.BarArea.ThemeColorName
      else
         this.parent.BarColour := this.parent.DefaultBarColour
      endif

      // Ensure overlayed EntryField position is always correct
      this.EF.left = this.left + 3

      // Ensure EntryField state matches the control
      if this.Disabled == true
         this.EF.enabled := false
      else
         this.EF.enabled := true
      endif


      local hDC, hDefPen, hDefBrush, xPaint, tw, th, cBMPName 

      local nCount, nTiles, nState

      tw = this.width
      th = this.height
       
      xPaint = space(32) // throw-away buffer
      
      // invalidate complete control to make repainting ready
      invalidateRect(this.hWND, 0, true)

      // Start the paint run
      hDC = beginpaint(this.hWND, xPaint)

      // Paint Background
      cBMPName = this.parent.BarColour + "_TOOLBARBACK_BMP"
      this.PaintBitmapStretch(cBMPName, hDC, 0, 0, tw, 22, 0, 1, 20, 22)
      
      // Paint EF Background
      if this.BackgroundName == ""
         cBackgroundName = "" 
      else
         cBackgroundName = this.parent.BarColour + "_" + this.BackgroundName            
      endif

      // How many tiles for the width
      nTiles = int(( tw - 8 ) / 23)

      // Get the State
      if this.IsSelected == true
         nState = 4 // Selected - State 4
      else
         if this.IsMouseOver == true 
            nState = 0 // Hot - state 0
         else
            nState = 1 // Normal - state 1
         endif
      endif

      // Paint EF Back
      this.PaintTransparentSubBitmap(cBackgroundName + "LEFT_BMP", hDC, 0, 0, "800080", 6, nState)
      for nCount = 1 to nTiles
         this.PaintTransparentSubBitmap(cBackgroundName + "BACK_BMP", hDC, 4 + (23 * nCount) - 23, 0, "800080", 6, nState)
      endfor // next
      this.PaintTransparentSubBitmap(cBackgroundName + "BACK_BMP", hDC, tw - 4 - 23, 0, "800080", 6, nState)
      this.PaintTransparentSubBitmap(cBackgroundName + "RIGHT_BMP", hDC, tw - 4, 0, "800080", 6, nState)

      // Right End ?
      if this.NoRightEnd == true
         this.PaintTransparentSubBitmap(cBackgroundName + "BACK_BMP", hDC, tw - 23, 0, "800080", 6, nState)
      endif

      // Finish Painting
      EndPaint(this.hWND,xPaint) 
   
   return

   // -------------------------------------

   function onMouseMove(flags, col, row)

      if this.Disabled == false

         if col > this.width or row > this.height or col < 0 or row < 0
            this.IsMouseOver := false
            this.IsMouseDown = false
            ReleaseCapture()
            class::TBEF_onPaint()
         else
            SetCapture(this.hwnd)
            if not this.IsMouseOver == true
               this.IsMouseOver := true
               class::TBEF_onPaint()
            endif
         endif

      endif

   return

   // -------------------------------------

   function TBEF_onLeftMousedown()

      if this.Disabled == false

         this.setFocus() 

         this.IsMouseOver := False

         ReleaseCapture()

         this.IsSelected := true

         this.EF.setFocus()

         // Paint this button
         class::TBEF_onPaint()

      endif

   return

   // -------------------------------------

   function EF_Key(nChar, nPosition, bShift, bControl)

      // Act on return key
      if nChar == 13
         try
            this.Owner.onMiddleDblClick()
         catch( exception e )
            // Event does not exist
         endtry
         return true
      endif

      // Permit only numeric
      if nChar < 48 or nChar >57
         ? chr(7)
         return false
      endif

   return true

   //--------------------------------------

endclass

//--------------------------------------------------------------------
//--------------------------------------------------------------------
//--------------------------------------------------------------------

Class rmToolBarTXT(oF) of rmPaintbox(oF) // from "rmPaintbox.cc"

   with (this)
      onPaint = class::ToolBarTXTonPaint
      top = 1
      width = 100
      height = 22
   endwith

   // -------------------------------------

   // Custom properties
   this.Resource = ""
   this.BackgroundName = ""
   this.Text = ""
	this.FontName = 'Tahoma' // 'Arial'
	this.FontSize = 9
	this.FontBold = false
	this.FontItalic = false
	this.FontUnderline = false
	this.FontStrikeout = false
	this.FontColour = ""  
   this.FontShadowColour = "" 
   this.TextAlign = 0
   this.Rotate = 0
   this.Emboss = true
   this.x = 5
   this.y = 5
   this.hModule = 0

   // -------------------------------------

   function CalcHeight(hdc)

   	// calculates the height of the font in units
      // corresponding to the output device
 	return MulDiv(this.fontSize, GetDeviceCaps(hDC, LOGPIXELSY), 72)


   // -------------------------------------

   function CreateFont(hdc, nRotate)

   	// returns a handle to a the font needed to draw the text
      return CreateFont(0-this.CalcHeight(hdc), 0, nRotate, nRotate, ;
         iif(this.fontBold, FW_BOLD,0), ;
         iif(this.fontItalic, CTRUE, CFALSE), ;
         iif(this.fontUnderline, CTRUE, CFALSE), ;
         iif(this.fontStrikeout, CTRUE, CFALSE), ;
         0, 0, 0, PROOF_QUALITY, 0, this.fontName)

   // -------------------------------------

   function ToolBarTXTonPaint()

      // Update toolbar colour if required
      if this.parent.BarArea.ThemeAware == true and this.parent.OverRideThemeColour == false
         this.parent.BarColour := this.parent.BarArea.ThemeColorName
      else
         this.parent.BarColour := this.parent.DefaultBarColour
      endif

      local hDC, hDefPen, hDefBrush, xPaint, tw, th, cBMPName
      local nFontColour, nFontShadowColour 
      local nCount, nTiles, nState

      tw = this.width
      th = this.height

      // Update Text colours with respect to bar colour.
      if this.FontColour == ""
         if this.parent.BarColour == "BLACKGLASS"
            nFontColour = 0xBBBBBB 
            nFontShadowColour = 0x444444 
         else
            nFontColour = 0x444444 
            nFontShadowColour = 0xEEEEEE
         endif
      endif
     
      xPaint = space(32) // throw-away buffer
      
      // Invalidate complete control to make repainting ready
      invalidateRect(this.hWND, 0, true)

      // Start the paint run
      hDC = beginpaint(this.hWND, xPaint)
      
      // Paint in toolbar background
      cBMPName = this.parent.BarColour + "_" + this.BackgroundName
      this.PaintBitmapStretch(cBMPName, hDC, 0, 0, tw, 22, 0, 1, 20, 22)

      // Store old TextColor
   	local n
      n = GetTextColor(hdc)

      // Set alignment
   	SetTextAlign(hdc, this.TextAlign)

      // Adjust vertical position for font size
      if this.FontSize > 9
         this.y := 5 - ((this.FontSize - 9) * 0.9)
      elseif this.FontSize < 9
         this.y := 5 + ((9 - this.FontSize) * 1.1)
      else
         this.y = 5
      endif

      // Draw text with transparent background
      SetBkMode(hdc, TRANSP)

		// Create the font to draw with      
      SelectObject(hdc, this.CreateFont(hdc, this.Rotate))

      // Underlay text
      if this.Emboss == true
         SetTextColor(hdc, nFontShadowColour)
         TextOut(hdc, this.x, this.y, this.Text, len(this.text))
      endif

      // Top text
      SetTextColor(hdc, nFontColour)
      TextOut(hdc, this.x-1, this.y-1, this.Text, len(this.text))

      // Delete the font
		DeleteObject(SelectObject(hdc, GetStockObject(SYSTEM_FONT)))

      // Restore Text Colour
      SetTextColor(hdc, n)

      // Finish Painting
      EndPaint(this.hWND,xPaint) 
   
   return

   // -------------------------------------

endclass

//--------------------------------------------------------------------
//--------------------------------------------------------------------
//--------------------------------------------------------------------

class EF_Index(oP) of ENTRYFIELD(oP) custom

   with (this)
      key = CLASS::EF_Key
      height = 14
      left = 0
      top = 5
      width = 46
      function = "I99999"
      border = false
      speedTip = "Enter the Patient's Index Number"
      value = "88888"   
      colorNormal = "black/blue" //white"
      borderStyle = 3   // None
      fontName = "Verdana"
      fontSize = 9.0
   endwith


   //--------------------------------------

   function EF_Key(nChar, nPosition, bShift, bControl)

      // Act on return key
      if nChar == 13 
         this.parent.PB.onClick()
      endif

   return true

   //--------------------------------------

   function onClick()

      local tp, nMax, cText

      tp = this.parent

      nMax = form.PxRowset.count()


      if val(tp.EF.value) > 0 and val(tp.EF.value) <= nMax

         form.PxRowset.findkey(str(val(tp.EF.value), 5, 0, "0"))

      else

         cText = "Valid Patient Index Numbers" + chr(13);
               + "are in the range :  1 to " + nMax
         msgBox(cText, " Invalid Input", 16 + 0)

      endif

      tp.EF.value = null
      tp.parent.GR_Docs.setfocus()

   return

   //--------------------------------------

endclass

//--------------------------------------------------------------------
//--------------------------------------------------------------------
//--------------------------------------------------------------------

// eof
